@@ -1,5 +1,5 @@
 /*
- * $Id: PdfPCell.java 3732 2009-02-25 17:41:43Z blowagie $
+ * $Id: PdfPCell.java 3992 2009-06-19 12:05:06Z blowagie $
  *
  * Copyright 2001, 2002 Paulo Soares
  *
@@ -52,7 +52,9 @@ package com.lowagie.text.pdf;
 import java.util.List;
 
 import com.lowagie.text.Chunk;
+import com.lowagie.text.DocumentException;
 import com.lowagie.text.Element;
+import com.lowagie.text.ExceptionConverter;
 import com.lowagie.text.Image;
 import com.lowagie.text.Phrase;
 import com.lowagie.text.Rectangle;
@@ -92,9 +94,15 @@ public class PdfPCell extends Rectangle{
     
     /** Holds value of property table. */
     private PdfPTable table;
-    
+
     /** Holds value of property colspan. */
     private int colspan = 1;
+
+    /**
+     * Holds value of property rowspan.
+     * @since	2.1.6
+     */
+    private int rowspan = 1;
     
     /** Holds value of property image. */
     private Image image;
@@ -211,6 +219,7 @@ public class PdfPCell extends Rectangle{
         	paddingTop = style.paddingTop;
         	paddingBottom = style.paddingBottom;
         	colspan = style.colspan;
+        	rowspan = style.rowspan;
         	cellEvent = style.cellEvent;
         	useDescender = style.useDescender;
         	useBorderPadding = style.useBorderPadding;
@@ -238,6 +247,7 @@ public class PdfPCell extends Rectangle{
         minimumHeight = cell.minimumHeight;
         noWrap = cell.noWrap;
         colspan = cell.colspan;
+        rowspan = cell.rowspan;
         if (cell.table != null)
             table = new PdfPTable(cell.table);
         image = Image.getInstance(cell.image);
@@ -644,7 +654,7 @@ public class PdfPCell extends Rectangle{
             table.setWidthPercentage(100);
         }
     }
-    
+
     /**
      * Getter for property colspan.
      * 
@@ -663,6 +673,26 @@ public class PdfPCell extends Rectangle{
         this.colspan = colspan;
     }
     
+    /**
+     * Getter for property rowspan.
+     * 
+     * @return Value of property rowspan.
+     * @since	2.1.6
+     */
+    public int getRowspan() {
+        return rowspan;
+    }
+    
+    /**
+     * Setter for property rowspan.
+     * 
+     * @param rowspan New value of property rowspan.
+     * @since	2.1.6
+     */
+    public void setRowspan(int rowspan) {
+        this.rowspan = rowspan;
+    }
+
     /**
      * Sets the following paragraph lines indent.
      * 
@@ -892,12 +922,92 @@ public class PdfPCell extends Rectangle{
      * @param rotation the rotation of the cell
      */
     public void setRotation(int rotation) {
+        rotation %= 360;
         if (rotation < 0)
             rotation += 360;
-        else
-            rotation %= 360;
         if ((rotation % 90) != 0)
             throw new IllegalArgumentException("Rotation must be a multiple of 90.");
         this.rotation = rotation;
     }
+    
+    /**
+     * Consumes part of the content of the cell.
+     * @param	height	the hight of the part that has to be consumed
+     * @since	2.1.6
+     */
+    void consumeHeight(float height) {
+        float rightLimit = getRight() - getEffectivePaddingRight();
+        float leftLimit = getLeft() + getEffectivePaddingLeft();
+        float bry = height - getEffectivePaddingTop() - getEffectivePaddingBottom();
+        if (getRotation() != 90 && getRotation() != 270) {
+            column.setSimpleColumn(leftLimit, bry + 0.001f,	rightLimit, 0);
+        }
+        else {
+        	column.setSimpleColumn(0, leftLimit, bry + 0.001f, rightLimit);
+        }
+        try {
+        	column.go(true);
+		} catch (DocumentException e) {
+			// do nothing
+		}
+    }
+	
+	/**
+	 * Returns the height of the cell.
+	 * @return	the height of the cell
+	 * @since	3.0.0
+	 */
+	public float getMaxHeight() {
+		boolean pivoted = (getRotation() == 90 || getRotation() == 270);
+		Image img = getImage();
+		if (img != null) {
+			img.scalePercent(100);
+			float refWidth = pivoted ? img.getScaledHeight() : img.getScaledWidth();
+			float scale = (getRight() - getEffectivePaddingRight()
+                    - getEffectivePaddingLeft() - getLeft()) / refWidth;
+			img.scalePercent(scale * 100);
+			float refHeight = pivoted ? img.getScaledWidth() : img.getScaledHeight();
+			setBottom(getTop() - getEffectivePaddingTop() - getEffectivePaddingBottom() - refHeight);
+		}
+		else {
+			if (pivoted && hasFixedHeight())
+				setBottom(getTop() - getFixedHeight());
+			else {
+				ColumnText ct = ColumnText.duplicate(getColumn());
+				float right, top, left, bottom;
+				if (pivoted) {
+					right = PdfPRow.RIGHT_LIMIT;
+					top = getRight() - getEffectivePaddingRight();
+					left = 0;
+					bottom = getLeft() + getEffectivePaddingLeft();
+				}
+				else {
+					right = isNoWrap() ? PdfPRow.RIGHT_LIMIT : getRight() - getEffectivePaddingRight();
+					top = getTop() - getEffectivePaddingTop();
+					left = getLeft() + getEffectivePaddingLeft();
+					bottom = hasFixedHeight() ? top + getEffectivePaddingBottom() - getFixedHeight() : PdfPRow.BOTTOM_LIMIT;
+				}
+				PdfPRow.setColumn(ct, left, bottom, right, top);
+				try {
+					ct.go(true);
+				} catch (DocumentException e) {
+					throw new ExceptionConverter(e);
+				}
+				if (pivoted)
+					setBottom(getTop() - getEffectivePaddingTop() - getEffectivePaddingBottom() - ct.getFilledWidth());
+				else {
+					float yLine = ct.getYLine();
+					if (isUseDescender())
+						yLine += ct.getDescender();
+					setBottom(yLine - getEffectivePaddingBottom());
+				}
+			}
+		}
+		float height = getHeight();
+		if (height < getFixedHeight())
+			height = getFixedHeight();
+		else if (height < getMinimumHeight())
+			height = getMinimumHeight();
+		return height;
+	}
 }
\ No newline at end of file
