@@ -1,5 +1,5 @@
 /*
- * $Id: PdfPRow.java 3777 2009-03-07 17:03:57Z blowagie $
+ * $Id: PdfPRow.java 3999 2009-06-30 11:52:55Z blowagie $
  *
  * Copyright 2001, 2002 Paulo Soares
  *
@@ -75,13 +75,19 @@ public class PdfPRow {
 	protected PdfPCell cells[];
 
 	protected float widths[];
+	
+	/**
+	 * extra heights that needs to be added to a cell because of rowspans.
+	 * @since	2.1.6
+	 */
+	protected float extraHeights[];
 
 	protected float maxHeight = 0;
-
+	
 	protected boolean calculated = false;
     
     private int[] canvasesPos;
-
+    
 	/**
 	 * Constructs a new PdfPRow with the cells in the array that was passed
 	 * as a parameter.
@@ -91,6 +97,7 @@ public class PdfPRow {
 	public PdfPRow(PdfPCell cells[]) {
 		this.cells = cells;
 		widths = new float[cells.length];
+		initExtraHeights();
 	}
 
 	/**
@@ -108,6 +115,7 @@ public class PdfPRow {
 		}
 		widths = new float[cells.length];
 		System.arraycopy(row.widths, 0, widths, 0, cells.length);
+		initExtraHeights();
 	}
 
 	/**
@@ -124,6 +132,12 @@ public class PdfPRow {
 		calculated = false;
 		for (int k = 0; k < widths.length; ++k) {
 			PdfPCell cell = cells[k];
+			
+			if (cell == null) {
+				total += widths[k];
+				continue;
+			}
+			
 			cell.setLeft(total);
 			int last = k + cell.getColspan();
 			for (; k < last; ++k)
@@ -135,6 +149,29 @@ public class PdfPRow {
 		return true;
 	}
 
+	/**
+	 * Initializes the extra heights array.
+	 * @since	2.1.6
+	 */
+	public void initExtraHeights() {
+		extraHeights = new float[cells.length];
+		for (int i = 0; i < extraHeights.length; i++) {
+			extraHeights[i] = 0;
+		}
+	}
+	
+	/**
+	 * Sets an extra height for a cell.
+	 * @param	cell	the index of the cell that needs an extra height
+	 * @param	height	the extra height
+	 * @since	2.1.6
+	 */
+	public void setExtraHeight(int cell, float height) {
+		if (cell < 0 || cell >= cells.length)
+			return;
+		extraHeights[cell] = height;
+	}
+	
 	/**
 	 * Calculates the heights of each cell in the row.
 	 * 
@@ -144,63 +181,15 @@ public class PdfPRow {
 		maxHeight = 0;
 		for (int k = 0; k < cells.length; ++k) {
 			PdfPCell cell = cells[k];
-			if (cell == null)
+			float height = 0;
+			if (cell == null) {
 				continue;
-			boolean pivoted = (cell.getRotation() == 90 || cell.getRotation() == 270);
-			Image img = cell.getImage();
-			if (img != null) {
-				img.scalePercent(100);
-				float refWidth = pivoted ? img.getScaledHeight() : img.getScaledWidth();
-				float scale = (cell.getRight() - cell.getEffectivePaddingRight()
-	                    - cell.getEffectivePaddingLeft() - cell.getLeft()) / refWidth;
-				img.scalePercent(scale * 100);
-				float refHeight = pivoted ? img.getScaledWidth() : img.getScaledHeight();
-				cell.setBottom(cell.getTop() - cell.getEffectivePaddingTop()
-						- cell.getEffectivePaddingBottom() - refHeight);
 			}
 			else {
-				if (pivoted && cell.hasFixedHeight())
-					cell.setBottom(cell.getTop() - cell.getFixedHeight());
-				else {
-					ColumnText ct = ColumnText.duplicate(cell.getColumn());
-					float right, top, left, bottom;
-                	if (pivoted) {
-                		right = RIGHT_LIMIT;
-                		top = cell.getRight() - cell.getEffectivePaddingRight();
-                		left = 0;
-                		bottom = cell.getLeft() + cell.getEffectivePaddingLeft();
-                	}
-                	else {
-                		right = cell.isNoWrap() ? RIGHT_LIMIT : cell.getRight() - cell.getEffectivePaddingRight();
-                		top = cell.getTop() - cell.getEffectivePaddingTop();
-                		left = cell.getLeft() + cell.getEffectivePaddingLeft();
-                		bottom = cell.hasFixedHeight() ? top + cell.getEffectivePaddingBottom() - cell.getFixedHeight() : BOTTOM_LIMIT;
-                	}
-                	setColumn(ct, left, bottom, right, top);
-					try {
-						ct.go(true);
-					} catch (DocumentException e) {
-						throw new ExceptionConverter(e);
-					}
-					if (pivoted)
-						cell.setBottom(cell.getTop() - cell.getEffectivePaddingTop() - cell.getEffectivePaddingBottom() - ct.getFilledWidth());
-					else {
-						float yLine = ct.getYLine();
-						if (cell.isUseDescender())
-							yLine += ct.getDescender();
-						cell.setBottom(yLine - cell.getEffectivePaddingBottom());
-					}
-				}
+				height = cell.getMaxHeight();
+				if ((height > maxHeight) && (cell.getRowspan() == 1))
+					maxHeight = height;
 			}
-			float height = cell.getFixedHeight();
-			if (height <= 0)
-				height = cell.getHeight();
-			if (height < cell.getFixedHeight())
-				height = cell.getFixedHeight();
-			else if (height < cell.getMinimumHeight())
-				height = cell.getMinimumHeight();
-			if (height > maxHeight)
-				maxHeight = height;
 		}
 		calculated = true;
 		return maxHeight;
@@ -211,17 +200,20 @@ public class PdfPRow {
 	 * 
 	 * @param xPos The x-coordinate where the table starts on the canvas
 	 * @param yPos The y-coordinate where the table starts on the canvas
+	 * @param currentMaxHeight The height of the cell to be drawn.
 	 * @param cell
 	 * @param canvases
+	 * @since	2.1.6	extra parameter currentMaxHeight
 	 */
-	public void writeBorderAndBackground(float xPos, float yPos, PdfPCell cell, PdfContentByte[] canvases) {
+	public void writeBorderAndBackground(float xPos, float yPos, float currentMaxHeight, PdfPCell cell, PdfContentByte[] canvases) {
 		Color background = cell.getBackgroundColor();
 		if (background != null || cell.hasBorders()) {
 			// Add xPos resp. yPos to the cell's coordinates for absolute coordinates
 			float right = cell.getRight() + xPos;
 			float top = cell.getTop() + yPos;
 			float left = cell.getLeft() + xPos;
-			float bottom = top - maxHeight;
+			float bottom = top - currentMaxHeight;
+			
 			if (background != null) {
 				PdfContentByte backgr = canvases[PdfPTable.BACKGROUNDCANVAS];
 				backgr.setColorFill(background);
@@ -240,7 +232,10 @@ public class PdfPRow {
 		}
 	}
 
-    private void saveAndRotateCanvases(PdfContentByte[] canvases, float a, float b, float c, float d, float e, float f) {
+	/**
+	 * @since	2.1.6 private is now protected
+	 */
+    protected void saveAndRotateCanvases(PdfContentByte[] canvases, float a, float b, float c, float d, float e, float f) {
         int last = PdfPTable.TEXTCANVAS + 1;
         if (canvasesPos == null)
             canvasesPos = new int[last * 2];
@@ -252,8 +247,11 @@ public class PdfPRow {
             canvasesPos[k * 2 + 1] = bb.size();
         }
     }
-    
-    private void restoreCanvases(PdfContentByte[] canvases) {
+
+	/**
+	 * @since	2.1.6 private is now protected
+	 */
+    protected void restoreCanvases(PdfContentByte[] canvases) {
         int last = PdfPTable.TEXTCANVAS + 1;
         for (int k = 0; k < last; ++k) {
             ByteBuffer bb = canvases[k].getInternalBuffer();
@@ -263,8 +261,11 @@ public class PdfPRow {
                 bb.setSize(canvasesPos[k * 2]);
         }
     }
-    
-    private float setColumn(ColumnText ct, float left, float bottom, float right, float top) {
+
+	/**
+	 * @since	3.0.0 protected is now public static
+	 */
+    public static float setColumn(ColumnText ct, float left, float bottom, float right, float top) {
         if (left > right)
             right = left;
         if (bottom > top)
@@ -295,33 +296,44 @@ public class PdfPRow {
 			colStart = 0;
 		if (colStart >= colEnd)
 			return;
+		
 		int newStart;
 		for (newStart = colStart; newStart >= 0; --newStart) {
 			if (cells[newStart] != null)
 				break;
-			xPos -= widths[newStart - 1];
+			if (newStart > 0)
+				xPos -= widths[newStart - 1];
 		}
-		xPos -= cells[newStart].getLeft();
+		
+		if (newStart < 0)
+			newStart = 0;
+		if (cells[newStart] != null)
+			xPos -= cells[newStart].getLeft();
 		
 		for (int k = newStart; k < colEnd; ++k) {
 			PdfPCell cell = cells[k];
 			if (cell == null)
 				continue;
-			writeBorderAndBackground(xPos, yPos, cell, canvases);
+			float currentMaxHeight = maxHeight + extraHeights[k];
+			
+			writeBorderAndBackground(xPos, yPos, currentMaxHeight, cell, canvases);
+
 			Image img = cell.getImage();
-			float tly = 0;
-			switch (cell.getVerticalAlignment()) {
-			case Element.ALIGN_BOTTOM:
-				tly = cell.getTop() + yPos - maxHeight + cell.getHeight()
-						- cell.getEffectivePaddingTop();
-				break;
-			case Element.ALIGN_MIDDLE:
-				tly = cell.getTop() + yPos + (cell.getHeight() - maxHeight) / 2
-						- cell.getEffectivePaddingTop();
-				break;
-			default:
-				tly = cell.getTop() + yPos - cell.getEffectivePaddingTop();
-				break;
+			
+			float tly = cell.getTop() + yPos - cell.getEffectivePaddingTop();
+			if (cell.getHeight() <= currentMaxHeight) {
+				switch (cell.getVerticalAlignment()) {
+				case Element.ALIGN_BOTTOM:
+					tly = cell.getTop() + yPos - currentMaxHeight + cell.getHeight()
+							- cell.getEffectivePaddingTop();
+					break;
+				case Element.ALIGN_MIDDLE:
+					tly = cell.getTop() + yPos + (cell.getHeight() - currentMaxHeight) / 2
+							- cell.getEffectivePaddingTop();
+					break;
+				default:
+					break;
+				}
 			}
 			if (img != null) {
                 if (cell.getRotation() != 0) {
@@ -329,9 +341,9 @@ public class PdfPRow {
                     img.setRotation(img.getImageRotation() + (float)(cell.getRotation() * Math.PI / 180.0));
                 }
 				boolean vf = false;
-				if (cell.getHeight() > maxHeight) {
+				if (cell.getHeight() > currentMaxHeight) {
 					img.scalePercent(100);
-					float scale = (maxHeight - cell.getEffectivePaddingTop() - cell
+					float scale = (currentMaxHeight - cell.getEffectivePaddingTop() - cell
 							.getEffectivePaddingBottom())
 							/ img.getScaledHeight();
 					img.scalePercent(scale * 100);
@@ -367,7 +379,7 @@ public class PdfPRow {
 			} else {
                 // rotation sponsored by Connection GmbH
                 if (cell.getRotation() == 90 || cell.getRotation() == 270) {
-                    float netWidth = maxHeight - cell.getEffectivePaddingTop() - cell.getEffectivePaddingBottom();
+                    float netWidth = currentMaxHeight - cell.getEffectivePaddingTop() - cell.getEffectivePaddingBottom();
                     float netHeight = cell.getWidth() - cell.getEffectivePaddingLeft() - cell.getEffectivePaddingRight();
                     ColumnText ct = ColumnText.duplicate(cell.getColumn());
                     ct.setCanvases(canvases);
@@ -385,11 +397,11 @@ public class PdfPRow {
                             calcHeight -= ct.getDescender();
                         ct = ColumnText.duplicate(cell.getColumn());
                         ct.setCanvases(canvases);
-                        ct.setSimpleColumn(0, -0.001f, netWidth + 0.001f, calcHeight);
+                        ct.setSimpleColumn(-0.003f, -0.001f, netWidth + 0.003f, calcHeight);
                         float pivotX;
                         float pivotY;
                         if (cell.getRotation() == 90) {
-                            pivotY = cell.getTop() + yPos - maxHeight + cell.getEffectivePaddingBottom();
+                            pivotY = cell.getTop() + yPos - currentMaxHeight + cell.getEffectivePaddingBottom();
                             switch (cell.getVerticalAlignment()) {
                             case Element.ALIGN_BOTTOM:
                                 pivotX = cell.getLeft() + xPos + cell.getWidth() - cell.getEffectivePaddingRight();
@@ -460,19 +472,19 @@ public class PdfPRow {
                     ColumnText ct = ColumnText.duplicate(cell.getColumn());
                     ct.setCanvases(canvases);
                     float bry = tly
-                            - (maxHeight /* cell.height() */
+                            - (currentMaxHeight
                             - cell.getEffectivePaddingTop() - cell.getEffectivePaddingBottom());
                     if (fixedHeight > 0) {
-                        if (cell.getHeight() > maxHeight) {
+                        if (cell.getHeight() > currentMaxHeight) {
                             tly = cell.getTop() + yPos - cell.getEffectivePaddingTop();
-                            bry = cell.getTop() + yPos - maxHeight + cell.getEffectivePaddingBottom();
+                            bry = cell.getTop() + yPos - currentMaxHeight + cell.getEffectivePaddingBottom();
                         }
                     }
                     if ((tly > bry || ct.zeroHeightElement()) && leftLimit < rightLimit) {
                         ct.setSimpleColumn(leftLimit, bry - 0.001f,	rightLimit, tly);
                         if (cell.getRotation() == 180) {
                             float shx = leftLimit + rightLimit;
-                            float shy = yPos + yPos - maxHeight + cell.getEffectivePaddingBottom() - cell.getEffectivePaddingTop();
+                            float shy = yPos + yPos - currentMaxHeight + cell.getEffectivePaddingBottom() - cell.getEffectivePaddingTop();
                             saveAndRotateCanvases(canvases, -1,0,0,-1,shx,shy);
                         }
                         try {
@@ -490,13 +502,13 @@ public class PdfPRow {
 			PdfPCellEvent evt = cell.getCellEvent();
 			if (evt != null) {
 				Rectangle rect = new Rectangle(cell.getLeft() + xPos, cell.getTop()
-						+ yPos - maxHeight, cell.getRight() + xPos, cell.getTop()
+						+ yPos - currentMaxHeight, cell.getRight() + xPos, cell.getTop()
 						+ yPos);
 				evt.cellLayout(cell, rect, canvases);
 			}
 		}
 	}
-
+	
 	/**
 	 * Checks if the dimensions of the columns were calculated.
 	 * 
@@ -552,19 +564,35 @@ public class PdfPRow {
 	 * The returned row is the remainder. It will return null if the newHeight
 	 * was so small that only an empty row would result.
 	 * 
-	 * @param newHeight	the new height
+	 * @param new_height	the new height
 	 * @return the remainder row or null if the newHeight was so small that only
 	 * an empty row would result
 	 */
-	public PdfPRow splitRow(float newHeight) {
+	public PdfPRow splitRow(PdfPTable table, int rowIndex, float new_height) {
 		PdfPCell newCells[] = new PdfPCell[cells.length];
 		float fixHs[] = new float[cells.length];
 		float minHs[] = new float[cells.length];
 		boolean allEmpty = true;
 		for (int k = 0; k < cells.length; ++k) {
+			float newHeight = new_height;
 			PdfPCell cell = cells[k];
-			if (cell == null)
+			if (cell == null) {
+				int index = rowIndex;
+				if (table.rowSpanAbove(index, k)) {
+					newHeight += table.getRowHeight(index);
+					while (table.rowSpanAbove(--index, k)) {
+						newHeight += table.getRowHeight(index);
+					}
+					PdfPRow row = table.getRow(index);
+					if (row != null && row.getCells()[k] != null) {
+						newCells[k] = new PdfPCell(row.getCells()[k]);
+						newCells[k].consumeHeight(newHeight);
+						newCells[k].setRowspan(row.getCells()[k].getRowspan() - rowIndex + index);
+						allEmpty = false;
+					}
+				}
 				continue;
+			}
 			fixHs[k] = cell.getFixedHeight();
 			minHs[k] = cell.getMinimumHeight();
 			Image img = cell.getImage();
@@ -622,7 +650,7 @@ public class PdfPRow {
 				if (fixHs[k] > 0)
 					cell.setFixedHeight(fixHs[k]);
 				else
-					cell.setMinimumHeight(minHs[k]);;
+					cell.setMinimumHeight(minHs[k]);
 			}
 			return null;
 		}
