@@ -74,13 +74,13 @@ public abstract class PdfContentStreamProcessor {
 	/** A map with all supported operators operators (PDF syntax). */
     private Map operators;
     /** Resources for the content stream. */
-    PdfDictionary resources;
+    private PdfDictionary resources;
     /** Stack keeping track of the graphics state. */
-    Stack gsStack = new Stack();
+    private Stack gsStack = new Stack();
     /** Text matrix. */
-    Matrix textMatrix;
+    private Matrix textMatrix;
     /** Text line matrix. */
-    Matrix textLineMatrix;
+    private Matrix textLineMatrix;
     
     /**
      * Creates a new PDF Content Stream Processor.
@@ -142,6 +142,24 @@ public abstract class PdfContentStreamProcessor {
         return (GraphicsState)gsStack.peek();
     }
     
+    /**
+     * Returns the current text matrix.
+     * @return	the text matrix
+     * @since 2.1.5
+     */
+    public Matrix getCurrentTextMatrix(){
+        return textMatrix;
+    }
+    
+    /**
+     * Returns the current line matrix.
+     * @return	the line matrix
+     * @since 2.1.5
+     */
+    public Matrix getCurrentTextLineMatrix(){
+        return textLineMatrix;
+    }
+    
     /**
      * Invokes an operator.
      * @param operator	the PDF Syntax of the operator
@@ -158,24 +176,15 @@ public abstract class PdfContentStreamProcessor {
     }
 
     /**
-     * Encodes a String based on the active font.
+     * Decodes a PdfString (which will contain glyph ids encoded in the font's encoding)
+     * based on the active font, and determine the unicode equivalent
      * @param in	the String that needs to be encoded
      * @return	the encoded String
+     * @since 2.1.7
      */
-    private String encode(String in){
+    private String decode(PdfString in){
         byte[] bytes = in.getBytes();
-        StringBuffer sb = new StringBuffer();
-        
-        for(int i = 0; i < bytes.length; i++){
-            String rslt = gs().font.encode(bytes, i, 1);
-            if (rslt == null){
-                rslt = gs().font.encode(bytes, i, 2);
-                i++;
-            }
-            sb.append(rslt);
-        }
-        
-        return sb.toString();
+        return gs().font.decode(bytes, 0, bytes.length);
     }
     
     /**
@@ -210,7 +219,7 @@ public abstract class PdfContentStreamProcessor {
      * @param tj		the text adjustment
      */
     public void displayPdfString(PdfString string, float tj){
-        String unicode = encode(string.toUnicodeString());
+        String unicode = decode(string);
         
         float width = getStringWidth(unicode, tj); // this is width in unscaled units - we have to normalize by the Tm scaling
 
@@ -252,9 +261,8 @@ public abstract class PdfContentStreamProcessor {
     private static class ShowTextArray implements ContentOperator{
         public void invoke(PdfContentStreamProcessor processor, PdfLiteral operator, ArrayList operands) {
             PdfArray array = (PdfArray)operands.get(0);
-            ArrayList entries = array.getArrayList();
             float tj = 0;
-            for (Iterator i = entries.iterator(); i.hasNext(); ) {
+            for (Iterator i = array.listIterator(); i.hasNext(); ) {
             	Object entryObj = i.next();
                 if (entryObj instanceof PdfString){
                     processor.displayPdfString((PdfString)entryObj, tj);
