@@ -1,5 +1,5 @@
 /*
- * $Id: RtfDestinationFontTable.java 3373 2008-05-12 16:21:24Z xlv $
+ * $Id: RtfDestinationFontTable.java 3832 2009-04-04 13:18:12Z blowagie $
  *
  * Copyright 2007 by Howard Shank (hgshank@yahoo.com)
  *
@@ -594,7 +594,8 @@ public final class RtfDestinationFontTable extends RtfDestination {
 		} else if ((operatingSystem.indexOf("nt") > -1)
 				|| (operatingSystem.indexOf("windows 2000") > -1)
 				|| (operatingSystem.indexOf("windows xp") > -1)
-				|| (operatingSystem.indexOf("windows 2003") > -1)) {
+				|| (operatingSystem.indexOf("windows 2003") > -1)
+				|| (operatingSystem.indexOf("windows vista") > -1)) {
 			process = runtime.exec("cmd.exe /c set");
 		} else {
 			process = runtime.exec("env");
